@@ -18,13 +18,11 @@
  */
 package org.codehaus.groovy.classgen.asm;
 
-import java.util.Iterator;
-import java.util.List;
-
 import org.codehaus.groovy.ast.ClassHelper;
 import org.codehaus.groovy.ast.ClassNode;
 import org.codehaus.groovy.ast.Parameter;
 import org.codehaus.groovy.ast.expr.ArgumentListExpression;
+import org.codehaus.groovy.ast.expr.BooleanExpression;
 import org.codehaus.groovy.ast.expr.ClosureListExpression;
 import org.codehaus.groovy.ast.expr.ConstantExpression;
 import org.codehaus.groovy.ast.expr.EmptyExpression;
@@ -52,7 +50,18 @@ import org.codehaus.groovy.classgen.asm.CompileStack.BlockRecorder;
 import org.objectweb.asm.Label;
 import org.objectweb.asm.MethodVisitor;
 
-import static org.objectweb.asm.Opcodes.*;
+import java.util.Iterator;
+import java.util.List;
+
+import static org.objectweb.asm.Opcodes.ALOAD;
+import static org.objectweb.asm.Opcodes.ATHROW;
+import static org.objectweb.asm.Opcodes.CHECKCAST;
+import static org.objectweb.asm.Opcodes.GOTO;
+import static org.objectweb.asm.Opcodes.IFEQ;
+import static org.objectweb.asm.Opcodes.MONITORENTER;
+import static org.objectweb.asm.Opcodes.MONITOREXIT;
+import static org.objectweb.asm.Opcodes.NOP;
+import static org.objectweb.asm.Opcodes.RETURN;
 
 public class StatementWriter {
     // iterator
@@ -159,7 +168,7 @@ public class StatementWriter {
         ClosureListExpression clExpr = (ClosureListExpression) loop.getCollectionExpression();
         controller.getCompileStack().pushVariableScope(clExpr.getVariableScope());
 
-        List expressions = clExpr.getExpressions();
+        List<Expression> expressions = clExpr.getExpressions();
         int size = expressions.size();
 
         // middle element is condition, lower half is init, higher half is increment
@@ -177,7 +186,7 @@ public class StatementWriter {
         mv.visitLabel(cond);
         // visit condition leave boolean on stack
         {
-            Expression condExpr = (Expression) expressions.get(condIndex);
+            Expression condExpr = expressions.get(condIndex);
             int mark = controller.getOperandStack().getStackLength();
             condExpr.visit(controller.getAcg());
             controller.getOperandStack().castToBool(mark,true);
@@ -228,10 +237,10 @@ public class StatementWriter {
         Label breakLabel = controller.getCompileStack().getBreakLabel();
 
         mv.visitLabel(continueLabel);
-        Expression bool = loop.getBooleanExpression();
+        BooleanExpression bool = loop.getBooleanExpression();
         boolean boolHandled = false;
-        if (bool instanceof ConstantExpression) {
-            ConstantExpression constant = (ConstantExpression) bool;
+        if (bool.getExpression() instanceof ConstantExpression) {
+            ConstantExpression constant = (ConstantExpression) bool.getExpression();
             if (constant.getValue()==Boolean.TRUE) {
                 boolHandled = true;
                 // do nothing
