@@ -16,27 +16,32 @@
 package org.gradle.api.internal;
 
 import groovy.lang.Closure;
-import groovy.lang.MissingPropertyException;
-import org.gradle.api.*;
-import org.gradle.api.internal.plugins.DefaultConvention;
-import org.gradle.api.plugins.Convention;
+import org.gradle.api.Action;
+import org.gradle.api.InvalidUserDataException;
+import org.gradle.api.NamedDomainObjectContainer;
+import org.gradle.api.Namer;
 import org.gradle.internal.Transformers;
+import org.gradle.internal.metaobject.AbstractDynamicObject;
+import org.gradle.internal.metaobject.BeanDynamicObject;
+import org.gradle.internal.metaobject.ConfigureDelegate;
+import org.gradle.internal.metaobject.DynamicObject;
+import org.gradle.internal.metaobject.GetPropertyResult;
+import org.gradle.internal.metaobject.InvokeMethodResult;
+import org.gradle.internal.metaobject.MixInClosurePropertiesAsMethodsDynamicObject;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.util.ConfigureUtil;
 
-import java.util.*;
+import java.util.Map;
 
 public abstract class AbstractPolymorphicDomainObjectContainer<T>
         extends AbstractNamedDomainObjectContainer<T> implements PolymorphicDomainObjectContainerInternal<T> {
 
     private final ContainerElementsDynamicObject elementsDynamicObject = new ContainerElementsDynamicObject();
-    private final Convention convention;
     private final DynamicObject dynamicObject;
 
     protected AbstractPolymorphicDomainObjectContainer(Class<T> type, Instantiator instantiator, Namer<? super T> namer) {
         super(type, instantiator, namer);
-        this.convention = new DefaultConvention(instantiator);
-        this.dynamicObject = new ExtensibleDynamicObject(this, new ContainerDynamicObject(elementsDynamicObject), convention);
+        this.dynamicObject = new ExtensibleDynamicObject(this, new ContainerDynamicObject(elementsDynamicObject), getConvention());
     }
 
     protected abstract <U extends T> U doCreate(String name, Class<U> type);
@@ -63,11 +68,6 @@ public abstract class AbstractPolymorphicDomainObjectContainer<T>
         return object;
     }
 
-    @Override
-    public Convention getConvention() {
-        return convention;
-    }
-
     @Override
     protected DynamicObject getElementsAsDynamicObject() {
         return elementsDynamicObject;
@@ -79,24 +79,24 @@ public abstract class AbstractPolymorphicDomainObjectContainer<T>
     }
 
     @Override
-    protected Object createConfigureDelegate(Closure configureClosure) {
-        return new PolymorphicDomainObjectContainerConfigureDelegate(configureClosure.getOwner(), this);
+    protected ConfigureDelegate createConfigureDelegate(Closure configureClosure) {
+        return new PolymorphicDomainObjectContainerConfigureDelegate(configureClosure, this);
     }
 
-    private class ContainerDynamicObject extends CompositeDynamicObject {
+    private class ContainerDynamicObject extends MixInClosurePropertiesAsMethodsDynamicObject {
         private ContainerDynamicObject(ContainerElementsDynamicObject elementsDynamicObject) {
             setObjects(new BeanDynamicObject(AbstractPolymorphicDomainObjectContainer.this), elementsDynamicObject, getConvention().getExtensionsAsDynamicObject());
         }
 
         @Override
-        protected String getDisplayName() {
+        public String getDisplayName() {
             return AbstractPolymorphicDomainObjectContainer.this.getDisplayName();
         }
     }
 
     private class ContainerElementsDynamicObject extends AbstractDynamicObject {
         @Override
-        protected String getDisplayName() {
+        public String getDisplayName() {
             return AbstractPolymorphicDomainObjectContainer.this.getDisplayName();
         }
 
@@ -106,12 +106,11 @@ public abstract class AbstractPolymorphicDomainObjectContainer<T>
         }
 
         @Override
-        public Object getProperty(String name) throws MissingPropertyException {
+        public void getProperty(String name, GetPropertyResult result) {
             Object object = findByName(name);
-            if (object == null) {
-                return super.getProperty(name);
+            if (object != null) {
+                result.result(object);
             }
-            return object;
         }
 
         @Override
@@ -125,16 +124,14 @@ public abstract class AbstractPolymorphicDomainObjectContainer<T>
         }
 
         @Override
-        public Object invokeMethod(String name, Object... arguments) throws groovy.lang.MissingMethodException {
+        public void invokeMethod(String name, InvokeMethodResult result, Object... arguments) {
             if (isConfigureMethod(name, arguments)) {
                 T element = getByName(name);
                 Object lastArgument = arguments[arguments.length - 1];
                 if (lastArgument instanceof Closure) {
                     ConfigureUtil.configure((Closure) lastArgument, element);
                 }
-                return element;
-            } else {
-                return super.invokeMethod(name, arguments);
+                result.result(element);
             }
         }
 
