@@ -28,9 +28,19 @@ import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.WeakHashMap;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
 
 public class JavaReflectionUtil {
+    private static final WeakHashMap<Class<?>, ConcurrentMap<String, Boolean>> PROPERTY_CACHE = new WeakHashMap<Class<?>, ConcurrentMap<String, Boolean>>();
+
     /**
      * Locates the readable properties of the given type. Searches only public properties.
      */
@@ -80,10 +90,8 @@ public class JavaReflectionUtil {
      * @throws NoSuchPropertyException
      */
     public static <T, F> PropertyAccessor<T, F> readableField(Class<T> target, Class<F> fieldType, String fieldName) throws NoSuchPropertyException {
-        Field field;
-        try {
-            field = target.getField(fieldName);
-        } catch (java.lang.NoSuchFieldException e) {
+        Field field = findField(target, fieldName);
+        if (field == null) {
             throw new NoSuchPropertyException(String.format("Could not find field '%s' on class %s.", fieldName, target.getSimpleName()));
         }
 
@@ -102,21 +110,17 @@ public class JavaReflectionUtil {
     }
 
     private static Method findGetterMethod(Class<?> target, String property) {
-        try {
-            Method getterMethod = target.getMethod(toMethodName("get", property));
-            if (isGetter(getterMethod)) {
-                return getterMethod;
+        Method[] methods = target.getMethods();
+        String getter = toMethodName("get", property);
+        String iser = toMethodName("is", property);
+        for (Method method : methods) {
+            String methodName = method.getName();
+            if (getter.equals(methodName) && isGetter(method)) {
+                return method;
             }
-        } catch (java.lang.NoSuchMethodException e) {
-            // Ignore
-        }
-        try {
-            Method getterMethod = target.getMethod(toMethodName("is", property));
-            if (isBooleanGetter(getterMethod)) {
-                return getterMethod;
+            if (iser.equals(methodName) && isBooleanGetter(method)) {
+                return method;
             }
-        } catch (java.lang.NoSuchMethodException e2) {
-            // Ignore
         }
         return null;
     }
@@ -171,13 +175,21 @@ public class JavaReflectionUtil {
      * @throws NoSuchPropertyException when the given property does not exist.
      */
     public static PropertyMutator writeableField(Class<?> target, String fieldName) throws NoSuchPropertyException {
-        Field field;
-        try {
-            field = target.getField(fieldName);
+        Field field = findField(target, fieldName);
+        if (field != null) {
             return new FieldBackedPropertyMutator(fieldName, field);
-        } catch (java.lang.NoSuchFieldException e) {
-            throw new NoSuchPropertyException(String.format("Could not find writeable field '%s' on class %s.", fieldName, target.getSimpleName()));
         }
+        throw new NoSuchPropertyException(String.format("Could not find writeable field '%s' on class %s.", fieldName, target.getSimpleName()));
+    }
+
+    private static Field findField(Class<?> target, String fieldName) {
+        Field[] fields = target.getFields();
+        for (Field field : fields) {
+            if (fieldName.equals(field.getName())) {
+                return field;
+            }
+        }
+        return null;
     }
 
     private static String toMethodName(String prefix, String propertyName) {
@@ -269,19 +281,27 @@ public class JavaReflectionUtil {
     // Not hasProperty() because that's awkward with Groovy objects implementing it
     public static boolean propertyExists(Object target, String propertyName) {
         Class<?> targetType = target.getClass();
+        ConcurrentMap<String, Boolean> cached;
+        synchronized (PROPERTY_CACHE) {
+            cached = PROPERTY_CACHE.get(targetType);
+            if (cached == null) {
+                cached = new ConcurrentHashMap<String, Boolean>();
+                PROPERTY_CACHE.put(targetType, cached);
+            }
+        }
+        Boolean res = cached.get(propertyName);
+        if (res != null) {
+            return res;
+        }
         Method getterMethod = findGetterMethod(target.getClass(), propertyName);
         if (getterMethod == null) {
-            try {
-                targetType.getField(propertyName);
-                return true;
-            } catch (NoSuchFieldException ignore) {
-                // ignore
+            if (findField(targetType, propertyName) == null) {
+                cached.putIfAbsent(propertyName, false);
+                return false;
             }
-        } else {
-            return true;
         }
-
-        return false;
+        cached.putIfAbsent(propertyName, true);
+        return true;
     }
 
     private static class MultiMap<K, V> extends HashMap<K, List<V>> {
@@ -380,7 +400,7 @@ public class JavaReflectionUtil {
 
         @Override
         public String toString() {
-            return String.format("property %s.%s", method.getDeclaringClass().getSimpleName(), property);
+            return "property " + method.getDeclaringClass().getSimpleName() + "." + property;
         }
 
         public String getName() {
@@ -444,7 +464,7 @@ public class JavaReflectionUtil {
 
         @Override
         public String toString() {
-            return String.format("property %s.%s", method.getDeclaringClass().getSimpleName(), property);
+            return "property " + method.getDeclaringClass().getSimpleName() + "." + property;
         }
 
         public String getName() {
@@ -477,7 +497,7 @@ public class JavaReflectionUtil {
 
         @Override
         public String toString() {
-            return String.format("field %s.%s", field.getDeclaringClass().getSimpleName(), name);
+            return "field " + field.getDeclaringClass().getSimpleName() + "." + name;
         }
 
         public String getName() {
