@@ -20,11 +20,17 @@ package groovy.json;
 
 import groovy.lang.Closure;
 import groovy.lang.DelegatesTo;
+import groovy.lang.GString;
 import groovy.lang.GroovyObjectSupport;
+import groovy.lang.Writable;
 
 import java.io.IOException;
 import java.io.Writer;
-import java.util.*;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
+import java.util.Map;
 
 /**
  * A builder for creating JSON payloads.
@@ -361,7 +367,7 @@ public class StreamingJsonBuilder extends GroovyObjectSupport {
      *     <li>or no argument at all</li>
      * </ul>
      * <p>
-     * Example with a classicala builder-style:
+     * Example with a classical builder-style:
      * <pre class="groovyTestCase">
      * new StringWriter().with { w ->
      *     def json = new groovy.json.StreamingJsonBuilder(w)
@@ -506,6 +512,9 @@ public class StreamingJsonBuilder extends GroovyObjectSupport {
                             if(value instanceof Closure) {
                                 call(name, (Closure)value);
                             }
+                            else if(value instanceof Writable) {
+                                call(name, (Writable)value);
+                            }
                             else {
                                 call(name, value);
                             }
@@ -651,6 +660,23 @@ public class StreamingJsonBuilder extends GroovyObjectSupport {
             writer.write(json.toString());
         }
 
+        /**
+         * Writes the given Writable as the value of the given attribute name
+         *
+         * @param name The attribute name 
+         * @param json The writable value
+         * @throws IOException
+         */
+        public void call(String name, Writable json) throws IOException {
+            writeName(name);
+            verifyValue();
+            if(json instanceof GString) {
+                writer.write(JsonOutput.toJson(json.toString()));
+            }
+            else {
+                json.writeTo(writer);
+            }
+        }
 
         private void writeObjects(Iterable coll, @DelegatesTo(StreamingJsonDelegate.class) Closure c) throws IOException {
             verifyValue();
