@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -93,7 +93,7 @@ class ExitCodeGenerators implements Iterable<ExitCodeGenerator> {
 				}
 			}
 			catch (Exception ex) {
-				exitCode = (exitCode == 0 ? 1 : exitCode);
+				exitCode = (exitCode != 0 ? exitCode : 1);
 				ex.printStackTrace();
 			}
 		}
