@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -106,8 +106,8 @@ public class ResourceBanner implements Banner {
 	}
 
 	protected String getApplicationVersion(Class<?> sourceClass) {
-		Package sourcePackage = (sourceClass == null ? null : sourceClass.getPackage());
-		return (sourcePackage == null ? null : sourcePackage.getImplementationVersion());
+		Package sourcePackage = (sourceClass != null ? sourceClass.getPackage() : null);
+		return (sourcePackage != null ? sourcePackage.getImplementationVersion() : null);
 	}
 
 	protected String getBootVersion() {
@@ -131,14 +131,14 @@ public class ResourceBanner implements Banner {
 		MutablePropertySources sources = new MutablePropertySources();
 		String applicationTitle = getApplicationTitle(sourceClass);
 		Map<String, Object> titleMap = Collections.<String, Object>singletonMap(
-				"application.title", (applicationTitle == null ? "" : applicationTitle));
+				"application.title", (applicationTitle != null ? applicationTitle : ""));
 		sources.addFirst(new MapPropertySource("title", titleMap));
 		return new PropertySourcesPropertyResolver(sources);
 	}
 
 	protected String getApplicationTitle(Class<?> sourceClass) {
-		Package sourcePackage = (sourceClass == null ? null : sourceClass.getPackage());
-		return (sourcePackage == null ? null : sourcePackage.getImplementationTitle());
+		Package sourcePackage = (sourceClass != null ? sourceClass.getPackage() : null);
+		return (sourcePackage != null ? sourcePackage.getImplementationTitle() : null);
 	}
 
 }
