@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.springframework.boot.bind;
 
+import java.beans.PropertyEditor;
 import java.net.InetAddress;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -35,6 +36,7 @@ import org.springframework.beans.InvalidPropertyException;
 import org.springframework.beans.MutablePropertyValues;
 import org.springframework.beans.NotWritablePropertyException;
 import org.springframework.beans.PropertyValue;
+import org.springframework.beans.propertyeditors.FileEditor;
 import org.springframework.core.convert.ConversionService;
 import org.springframework.core.convert.TypeDescriptor;
 import org.springframework.core.env.StandardEnvironment;
@@ -58,6 +60,13 @@ import org.springframework.validation.DataBinder;
  */
 public class RelaxedDataBinder extends DataBinder {
 
+	private static final Set<Class<?>> EXCLUDED_EDITORS;
+	static {
+		Set<Class<?>> excluded = new HashSet<Class<?>>();
+		excluded.add(FileEditor.class);
+		EXCLUDED_EDITORS = Collections.unmodifiableSet(excluded);
+	}
+
 	private static final Object BLANK = new Object();
 
 	private String namePrefix;
@@ -385,7 +394,7 @@ public class RelaxedDataBinder extends DataBinder {
 		if (propertyName == null) {
 			propertyName = resolveNestedPropertyName(target, prefix, name);
 		}
-		return (propertyName == null ? name : propertyName);
+		return (propertyName != null ? propertyName : name);
 	}
 
 	private String resolveNestedPropertyName(BeanWrapper target, String prefix,
@@ -453,6 +462,24 @@ public class RelaxedDataBinder extends DataBinder {
 		return target;
 	}
 
+	@Override
+	public void registerCustomEditor(Class<?> requiredType,
+			PropertyEditor propertyEditor) {
+		if (propertyEditor == null
+				|| !EXCLUDED_EDITORS.contains(propertyEditor.getClass())) {
+			super.registerCustomEditor(requiredType, propertyEditor);
+		}
+	}
+
+	@Override
+	public void registerCustomEditor(Class<?> requiredType, String field,
+			PropertyEditor propertyEditor) {
+		if (propertyEditor == null
+				|| !EXCLUDED_EDITORS.contains(propertyEditor.getClass())) {
+			super.registerCustomEditor(requiredType, field, propertyEditor);
+		}
+	}
+
 	/**
 	 * Holder to allow Map targets to be bound.
 	 */
@@ -712,7 +739,7 @@ public class RelaxedDataBinder extends DataBinder {
 		}
 
 		private boolean isBenign(PropertyOrigin origin) {
-			String name = (origin == null ? null : origin.getSource().getName());
+			String name = (origin != null ? origin.getSource().getName() : null);
 			return BENIGN_PROPERTY_SOURCE_NAMES.contains(name);
 		}
 
