@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -87,8 +87,8 @@ public class YamlJavaBeanPropertyConstructor extends Constructor {
 				throws IntrospectionException {
 			Map<String, Property> forType = YamlJavaBeanPropertyConstructor.this.properties
 					.get(type);
-			Property property = (forType == null ? null : forType.get(name));
-			return (property == null ? super.getProperty(type, name) : property);
+			Property property = (forType != null ? forType.get(name) : null);
+			return (property != null ? property : super.getProperty(type, name));
 		}
 
 	}
