@@ -131,9 +131,10 @@ abstract class ArchiveCommand extends OptionParsingCommand {
 
 			File output = new File((String) nonOptionArguments.remove(0));
 			Assert.isTrue(
-					output.getName().toLowerCase(Locale.ENGLISH).endsWith("." + this.type),
-					"The output '" + output + "' is not a " + this.type.toUpperCase(
-							Locale.ENGLISH) + " file.");
+					output.getName().toLowerCase(Locale.ENGLISH)
+							.endsWith("." + this.type),
+					"The output '" + output + "' is not a "
+							+ this.type.toUpperCase(Locale.ENGLISH) + " file.");
 			deleteIfExists(output);
 
 			GroovyCompiler compiler = createCompiler(options);
@@ -238,7 +239,7 @@ abstract class ArchiveCommand extends OptionParsingCommand {
 		private String commaDelimitedClassNames(Class<?>[] classes) {
 			StringBuilder builder = new StringBuilder();
 			for (int i = 0; i < classes.length; i++) {
-				builder.append(i == 0 ? "" : ",");
+				builder.append(i != 0 ? "," : "");
 				builder.append(classes[i].getName());
 			}
 			return builder.toString();
