@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2017 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -149,8 +149,8 @@ class InitializrServiceMetadata {
 		}
 		JSONObject type = root.getJSONObject(TYPE_EL);
 		JSONArray array = type.getJSONArray(VALUES_EL);
-		String defaultType = type.has(DEFAULT_ATTRIBUTE)
-				? type.getString(DEFAULT_ATTRIBUTE) : null;
+		String defaultType = (type.has(DEFAULT_ATTRIBUTE)
+				? type.getString(DEFAULT_ATTRIBUTE) : null);
 		for (int i = 0; i < array.length(); i++) {
 			JSONObject typeJson = array.getJSONObject(i);
 			ProjectType projectType = parseType(typeJson, defaultType);
@@ -212,7 +212,7 @@ class InitializrServiceMetadata {
 
 	private String getStringValue(JSONObject object, String name, String defaultValue)
 			throws JSONException {
-		return object.has(name) ? object.getString(name) : defaultValue;
+		return (object.has(name) ? object.getString(name) : defaultValue);
 	}
 
 	private Map<String, String> parseStringItems(JSONObject json) throws JSONException {
@@ -227,7 +227,7 @@ class InitializrServiceMetadata {
 		return result;
 	}
 
-	private final static class MetadataHolder<K, T> {
+	private static final class MetadataHolder<K, T> {
 
 		private final Map<K, T> content;
 
