@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -164,7 +164,7 @@ public class OptionHandler {
 		OptionHelpAdapter(OptionDescriptor descriptor) {
 			this.options = new LinkedHashSet<String>();
 			for (String option : descriptor.options()) {
-				this.options.add((option.length() == 1 ? "-" : "--") + option);
+				this.options.add((option.length() != 1 ? "--" : "-") + option);
 			}
 			if (this.options.contains("--cp")) {
 				this.options.remove("--cp");
