@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2014 the original author or authors.
+ * Copyright 2012-2017 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -140,7 +140,7 @@ public class RunCommand extends OptionParsingCommand {
 
 			@Override
 			public Level getLogLevel() {
-				if (getOptions().has(RunOptionHandler.this.quietOption)) {
+				if (isQuiet()) {
 					return Level.OFF;
 				}
 				if (getOptions().has(RunOptionHandler.this.verboseOption)) {
@@ -149,6 +149,11 @@ public class RunCommand extends OptionParsingCommand {
 				return Level.INFO;
 			}
 
+			@Override
+			public boolean isQuiet() {
+				return getOptions().has(RunOptionHandler.this.quietOption);
+			}
+
 		}
 
 	}
