@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -53,7 +53,7 @@ public class ShellPrompts {
 	 * @return the current prompt
 	 */
 	public String getPrompt() {
-		return this.prompts.isEmpty() ? DEFAULT_PROMPT : this.prompts.peek();
+		return (this.prompts.isEmpty() ? DEFAULT_PROMPT : this.prompts.peek());
 	}
 
 }
