@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -222,8 +222,8 @@ public class DependencyManagementBomTransformation
 				return new UrlModelSource(
 						Grape.getInstance().resolve(null, dependency)[0].toURL());
 			}
-			catch (MalformedURLException e) {
-				throw new UnresolvableModelException(e.getMessage(), groupId, artifactId,
+			catch (MalformedURLException ex) {
+				throw new UnresolvableModelException(ex.getMessage(), groupId, artifactId,
 						version);
 			}
 		}
