@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,9 +38,8 @@ public class SpringSecurityCompilerAutoConfiguration extends CompilerAutoConfigu
 
 	@Override
 	public void applyDependencies(DependencyCustomizer dependencies) {
-		dependencies
-				.ifAnyMissingClasses(
-						"org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity")
+		dependencies.ifAnyMissingClasses(
+				"org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity")
 				.add("spring-boot-starter-security");
 	}
 
