@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2016 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -116,8 +116,8 @@ public class MavenSettingsReader {
 			try {
 				this._cipher = new DefaultPlexusCipher();
 			}
-			catch (PlexusCipherException e) {
-				throw new IllegalStateException(e);
+			catch (PlexusCipherException ex) {
+				throw new IllegalStateException(ex);
 			}
 		}
 
