@@ -73,11 +73,11 @@ public abstract class AbstractEmbeddedServletContainerFactory
 	protected final File getValidDocumentRoot() {
 		File file = getDocumentRoot();
 		// If document root not explicitly set see if we are running from a war archive
-		file = file != null ? file : getWarFileDocumentRoot();
+		file = (file != null ? file : getWarFileDocumentRoot());
 		// If not a war archive maybe it is an exploded war
-		file = file != null ? file : getExplodedWarFileDocumentRoot();
+		file = (file != null ? file : getExplodedWarFileDocumentRoot());
 		// Or maybe there is a document root in a well-known location
-		file = file != null ? file : getCommonDocumentRoot();
+		file = (file != null ? file : getCommonDocumentRoot());
 		if (file == null && this.logger.isDebugEnabled()) {
 			this.logger
 					.debug("None of the document roots " + Arrays.asList(COMMON_DOC_ROOTS)
@@ -109,7 +109,7 @@ public abstract class AbstractEmbeddedServletContainerFactory
 	private boolean isStaticResourceJar(URL url) {
 		try {
 			if ("file".equals(url.getProtocol())) {
-				File file = new File(getDecodedFile(url));
+				File file = new File(url.toURI());
 				return (file.isDirectory()
 						&& new File(file, "META-INF/resources").isDirectory())
 						|| isResourcesJar(file);
@@ -122,12 +122,19 @@ public abstract class AbstractEmbeddedServletContainerFactory
 				}
 			}
 		}
-		catch (IOException ex) {
+		catch (Exception ex) {
 			throw new IllegalStateException(ex);
 		}
 		return false;
 	}
 
+	/**
+	 * Converts the given {@code url} into a decoded file path.
+	 * @param url the url to convert
+	 * @return the file path
+	 * @deprecated Since 1.5.13 in favor of {@link File#File(java.net.URI)}
+	 */
+	@Deprecated
 	protected final String getDecodedFile(URL url) {
 		try {
 			return URLDecoder.decode(url.getFile(), "UTF-8");
@@ -217,7 +224,7 @@ public abstract class AbstractEmbeddedServletContainerFactory
 
 	File getCodeSourceArchive(CodeSource codeSource) {
 		try {
-			URL location = (codeSource == null ? null : codeSource.getLocation());
+			URL location = (codeSource != null ? codeSource.getLocation() : null);
 			if (location == null) {
 				return null;
 			}
