@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2015 the original author or authors.
+ * Copyright 2012-2018 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -109,7 +109,7 @@ public class SpringApplicationJsonEnvironmentPostProcessor
 
 	private void flatten(String prefix, Map<String, Object> result,
 			Map<String, Object> map) {
-		prefix = (prefix == null ? "" : prefix + ".");
+		prefix = (prefix != null ? prefix + "." : "");
 		for (Map.Entry<String, Object> entry : map.entrySet()) {
 			extract(prefix + entry.getKey(), result, entry.getValue());
 		}
