@@ -618,14 +618,14 @@ public class RestTemplateBuilder {
 
 	private <T> Set<T> append(Set<T> set, T addition) {
 		Set<T> result = new LinkedHashSet<T>(
-				set == null ? Collections.<T>emptySet() : set);
+				set != null ? set : Collections.<T>emptySet());
 		result.add(addition);
 		return Collections.unmodifiableSet(result);
 	}
 
 	private <T> Set<T> append(Set<T> set, Collection<? extends T> additions) {
 		Set<T> result = new LinkedHashSet<T>(
-				set == null ? Collections.<T>emptySet() : set);
+				set != null ? set : Collections.<T>emptySet());
 		result.addAll(additions);
 		return Collections.unmodifiableSet(result);
 	}
@@ -642,7 +642,7 @@ public class RestTemplateBuilder {
 	/**
 	 * {@link RequestFactoryCustomizer} to call a "set timeout" method.
 	 */
-	private static abstract class TimeoutRequestFactoryCustomizer
+	private abstract static class TimeoutRequestFactoryCustomizer
 			implements RequestFactoryCustomizer {
 
 		private final int timeout;
