@@ -368,7 +368,7 @@ public class ReflectionHelper {
 	}
 
 
-	static enum ArgumentsMatchKind {
+	enum ArgumentsMatchKind {
 
 		/** An exact match is where the parameter types exactly match what the method/constructor is expecting */
 		EXACT,
