@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2016 the original author or authors.
+ * Copyright 2002-2017 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,6 +35,7 @@ import java.util.regex.Pattern;
  * <li>{@code ?} matches one character</li>
  * <li>{@code *} matches zero or more characters</li>
  * <li>{@code **} matches zero or more <em>directories</em> in a path</li>
+ * <li>{@code {spring:[a-z]+}} matches the regexp {@code [a-z]+} as a path variable named "spring"</li>
  * </ul>
  *
  * <h3>Examples</h3>
@@ -50,6 +51,8 @@ import java.util.regex.Pattern;
  * <li><code>org/&#42;&#42;/servlet/bla.jsp</code> &mdash; matches
  * {@code org/springframework/servlet/bla.jsp} but also
  * {@code org/springframework/testing/servlet/bla.jsp} and {@code org/servlet/bla.jsp}</li>
+ * <li>{@code com/{filename:\\w+}.jsp} will match {@code com/test.jsp} and assign the value {@code test}
+ * to the {@code filename} variable</li>
  * </ul>
  *
  * <p><strong>Note:</strong> a pattern and a path must both be absolute or must
@@ -217,8 +220,7 @@ public class AntPathMatcher implements PathMatcher {
 		if (pathIdxStart > pathIdxEnd) {
 			// Path is exhausted, only match if rest of pattern is * or **'s
 			if (pattIdxStart > pattIdxEnd) {
-				return (pattern.endsWith(this.pathSeparator) ? path.endsWith(this.pathSeparator) :
-						!path.endsWith(this.pathSeparator));
+				return (pattern.endsWith(this.pathSeparator) == path.endsWith(this.pathSeparator));
 			}
 			if (!fullMatch) {
 				return true;
@@ -315,17 +317,13 @@ public class AntPathMatcher implements PathMatcher {
 
 	private boolean isPotentialMatch(String path, String[] pattDirs) {
 		if (!this.trimTokens) {
-			char[] pathChars = path.toCharArray();
 			int pos = 0;
 			for (String pattDir : pattDirs) {
 				int skipped = skipSeparator(path, pos, this.pathSeparator);
 				pos += skipped;
-				skipped = skipSegment(pathChars, pos, pattDir);
+				skipped = skipSegment(path, pos, pattDir);
 				if (skipped < pattDir.length()) {
-					if (skipped > 0) {
-						return true;
-					}
-					return (pattDir.length() > 0) && isWildcardChar(pattDir.charAt(0));
+					return (skipped > 0 || (pattDir.length() > 0 && isWildcardChar(pattDir.charAt(0))));
 				}
 				pos += skipped;
 			}
@@ -333,16 +331,18 @@ public class AntPathMatcher implements PathMatcher {
 		return true;
 	}
 
-	private int skipSegment(char[] chars, int pos, String prefix) {
+	private int skipSegment(String path, int pos, String prefix) {
 		int skipped = 0;
-		for (char c : prefix.toCharArray()) {
+		for (int i = 0; i < prefix.length(); i++) {
+			char c = prefix.charAt(i);
 			if (isWildcardChar(c)) {
 				return skipped;
 			}
-			else if (pos + skipped >= chars.length) {
+			int currPos = pos + skipped;
+			if (currPos >= path.length()) {
 				return 0;
 			}
-			else if (chars[pos + skipped] == c) {
+			if (c == path.charAt(currPos)) {
 				skipped++;
 			}
 		}
